/** @LICENSE
 * @hi-ui/ellipsis-tooltip
 * https://github.com/XiaoMi/hiui/tree/master/packages/ui/ellipsis-tooltip#readme
 *
 * Copyright (c) HiUI <mi-hiui@xiaomi.com>.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
'use strict';

var _typeof = require("@babel/runtime/helpers/typeof");
Object.defineProperty(exports, '__esModule', {
  value: true
});
var React = require('react');
var classname = require('@hi-ui/classname');
var env = require('@hi-ui/env');
var Tooltip = require('@hi-ui/tooltip');
function _interopDefaultCompat(e) {
  return e && _typeof(e) === 'object' && 'default' in e ? e : {
    'default': e
  };
}
var React__default = /*#__PURE__*/_interopDefaultCompat(React);
var Tooltip__default = /*#__PURE__*/_interopDefaultCompat(Tooltip);
var ELLIPSIS_TOOLTIP_PREFIX = classname.getPrefixCls('ellipsis-tooltip');
// 格式化子文本，超出规定字数时展示...
var formatChildText = function formatChildText(text, maxTextCount) {
  var stringText = text === null || text === void 0 ? void 0 : text.toString();
  var canTooltip = maxTextCount > 0 && stringText.length > maxTextCount;
  return canTooltip ? stringText.slice(0, maxTextCount) + "..." : text;
};
/**
 * 这是一个文字超出后，展示省略号，并且鼠标悬停时出现全部描述的组件
 */
var EllipsisTooltip = function EllipsisTooltip(_ref) {
  var _cx;
  var _ref$prefixCls = _ref.prefixCls,
    prefixCls = _ref$prefixCls === void 0 ? ELLIPSIS_TOOLTIP_PREFIX : _ref$prefixCls,
    _ref$role = _ref.role,
    role = _ref$role === void 0 ? 'ellipsis-tooltip' : _ref$role,
    className = _ref.className,
    children = _ref.children,
    _ref$numberOfLines = _ref.numberOfLines,
    numberOfLines = _ref$numberOfLines === void 0 ? 1 : _ref$numberOfLines,
    _ref$maxTextCount = _ref.maxTextCount,
    maxTextCount = _ref$maxTextCount === void 0 ? 0 : _ref$maxTextCount,
    tooltipProps = _ref.tooltipProps;
  var _useState = React.useState(true),
    disableTooltip = _useState[0],
    setDisableTooltip = _useState[1];
  var contentRef = React.useRef(null);
  // 检查文本超出省略号
  var handleCheckEllipsis = React.useCallback(function () {
    var _a, _b, _c;
    if (contentRef.current) {
      // 多行超出
      if (numberOfLines > 1) {
        var style = window.getComputedStyle(contentRef.current, null);
        var fontSize = style.fontSize;
        var lineHeight = style.lineHeight === 'normal' ? +fontSize : +style.lineHeight;
        var textLines = Math.round(+style.height / lineHeight);
        setDisableTooltip(textLines <= numberOfLines);
      } else {
        // 单行超出
        var rect = (_a = contentRef.current) === null || _a === void 0 ? void 0 : _a.getBoundingClientRect();
        var parentRect = (_c = (_b = contentRef.current) === null || _b === void 0 ? void 0 : _b.parentNode) === null || _c === void 0 ? void 0 : _c.getBoundingClientRect();
        setDisableTooltip(+(rect === null || rect === void 0 ? void 0 : rect.width) <= parentRect.width);
      }
    }
  }, [numberOfLines]);
  var update = React.useCallback(function () {
    // 当文字字数超出有配置时，单独处理
    if (maxTextCount > 0 && maxTextCount < (children === null || children === void 0 ? void 0 : children.length)) {
      setDisableTooltip(false);
    } else {
      handleCheckEllipsis();
    }
  }, [children === null || children === void 0 ? void 0 : children.length, handleCheckEllipsis, maxTextCount]);
  React.useEffect(function () {
    var _a;
    update();
    var observer = new ResizeObserver(function () {
      // fix: https://github.com/XiaoMi/hiui/issues/2764
      // 在有动画(300ms)的的组件中使用该组件会导致计算有误，此处做兼容处理
      setTimeout(function () {
        update();
      }, 300);
    });
    ((_a = contentRef.current) === null || _a === void 0 ? void 0 : _a.parentNode) && observer.observe(contentRef.current.parentNode);
    return function () {
      observer.disconnect();
    };
  }, [update]);
  var cls = classname.cx(prefixCls, className, (_cx = {}, _cx[prefixCls + "--single"] = maxTextCount === 0 && (!numberOfLines || numberOfLines <= 1), _cx[prefixCls + "--multiple"] = numberOfLines > 1, _cx), prefixCls + "--" + (disableTooltip ? 'disabled' : 'available'));
  return /*#__PURE__*/React__default["default"].createElement(Tooltip__default["default"], Object.assign({
    disabled: disableTooltip
  }, Object.assign({
    title: children
  }, tooltipProps)), /*#__PURE__*/React__default["default"].createElement("div", {
    role: role,
    className: cls,
    style: {
      WebkitLineClamp: numberOfLines > 1 ? numberOfLines : undefined
    }
  }, /*#__PURE__*/React__default["default"].createElement("span", {
    ref: contentRef
  }, formatChildText(children, maxTextCount))));
};
if (env.__DEV__) {
  EllipsisTooltip.displayName = 'EllipsisTooltip';
}
exports.EllipsisTooltip = EllipsisTooltip;
